/*
 * i2c-soft.c
 *
 *  Created on: 19.12.2009
 *      Author: Pavel V. Gololobov
 */
#if defined (NRF51)
		#include "nrf51.h"
#elif defined (NRF52)
		#include "nrf52.h"
#endif

#include "nrf_gpio.h"
#include "i2c.h"

//------------------------------------------------------------------
// I2C Speed Down
//------------------------------------------------------------------
#define I2CWAIT i2c_Wait(15) //100 kHz operations

//------------------------------------------------------------------
// I2C Delay
//------------------------------------------------------------------
void i2c_Wait(int32_t n)
{
	do
	{
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
	}
	while(--n);
}

//------------------------------------------------------------------
// I2C SDA SCL Control
//------------------------------------------------------------------
void SetLowSDA(void)
{
	nrf_gpio_pin_clear(SDA);
	I2CWAIT;
}
void SetHighSDA(void)
{
	nrf_gpio_pin_set(SDA);
	I2CWAIT;
}

void SetLowSCL(void)
{
	nrf_gpio_pin_clear(SCL);
	I2CWAIT;
}
void SetHighSCL(void)
{
	nrf_gpio_pin_set(SCL);
	I2CWAIT;
}

//------------------------------------------------------------------
// I2C Initialize Bus
//------------------------------------------------------------------
void i2c_Init(void)
{

	nrf_gpio_pin_clear(SCL);
	nrf_gpio_pin_clear(SDA);
	
	nrf_gpio_cfg_output(SDA);
	nrf_gpio_cfg_output(SCL);
	
	//nrf_gpio_cfg_input(SDA, NRF_GPIO_PIN_NOPULL);
	SetHighSCL();
//	SetLowSDA();
	SetHighSDA();
}

//------------------------------------------------------------------
// I2C Start Data Transfer
//------------------------------------------------------------------
void i2c_Start(void)
{
	SetHighSCL();
	SetHighSDA();

	SetHighSCL();
	SetLowSDA();

	SetLowSCL();
	SetHighSDA();
}

//------------------------------------------------------------------
// I2C Stop  Transfer
//------------------------------------------------------------------
void i2c_Stop(void)
{
	SetLowSCL();
	SetLowSDA();

	SetHighSCL();
	SetLowSDA();

	SetHighSCL();
	SetHighSDA();
}

//------------------------------------------------------------------
// I2C Write  Transfer
//------------------------------------------------------------------
int16_t i2c_Write(uint8_t a)
{
	int16_t i;
	int16_t return_ack;

	for (i = 0; i < 8; i++)
    {
		SetLowSCL();
		if (a & 0x80)
			SetHighSDA();
		else
			SetLowSDA();

		SetHighSCL();
		a <<= 1;
	}
	SetLowSCL();

	/* ack Read */
	SetHighSDA();
	SetHighSCL();

	if ((uint8_t)(nrf_gpio_pin_read(SDA)))
		return_ack = NO_I2C_ACK;
	else
		return_ack = OK_I2C_ACK;

    SetLowSCL();

	return (return_ack);
}

//------------------------------------------------------------------
// I2C Read  Transfer
//------------------------------------------------------------------
uint8_t i2c_Read(int16_t ack)
{
	int16_t i;
	uint8_t caracter = 0x00;

	SetLowSCL();
	SetHighSDA();

	for (i = 0; i < 8; i++)
    {
		caracter = caracter << 1;
		SetHighSCL();
		if ((uint8_t)(nrf_gpio_pin_read(SDA)))
			caracter = caracter  + 1;

		SetLowSCL();
	}

	if (ack)
    {
		SetLowSDA();
	}
	SetHighSCL();
	SetLowSCL();

	return (caracter);
}
